#!/bin/bash
#
# Deploy currently built platforms based on the deploy.yml
# configuration, or another file if passed.
#
# Usage:
#   ./script/deploy.sh [deploy.yml]
#
# Note:
#   * platforms will not be built if they do not exist
#   * deployment via rsync/ssh is assumed
#   * no pre-deployment or post-deployment checks are performed
#
# An alternative timestamp for the deployments may be passed
# to the script through the DEPLOY_DATE environments variable.
# Eg.  DEPLOY_DATE=2047.01.01 ./script/deploy.sh
#

# TODO: either trash script completely
# OR maybe translate in python ? https://stackoverflow.com/a/20114807
# OR just keep it barebone
CONFIG_FILE="$PWD/deploy.yml"
DEPLOY_DATE=`date +'%Y.%m.%d'`
DEST=build
BUILD_NUMBER=


# Iterate over hosts
for i in "${hosts_[@]}" ; do
    echo "Host: $i"
    deploy_host "$i"
done

function deploy_host() {
    local host=$1
    local hostname="${!hn_key}"
    local user="${!u_key}"
    local basepath="${!bp_key}"
    local aegir_provision="${!aegir_provision_key}"
    local -n platforms="$p_key"
    for i in "${platforms[@]}" ; do
        local basename=${!p_basename_var}
        echo -ne "\tPlatform: $i... "
        if [ ! -d "${DEST}/${i}-${BUILD_NUMBER}" ] ; then
            echo "fail! Directory ${DEST}/${i}-${BUILD_NUMBER} does not exist."
            continue
        fi
        # TODO: THIS IS IT!!
        CURRENT_DATE="$DEPLOY_DATE" PLATFORM="$i" DEPLOY_USER="$user" \
        DEPLOY_HOST="$hostname" DEPLOY_PATH="$basepath" DEST="$DEST" \
        BUILD_NUMBER="$BUILD_NUMBER" DEPLOY_BASENAME="$basename" make \
        deploy-platform && echo "OK" || echo "FAILED: Returned $?"

        if [ "$aegir_provision" ] ; then
            # /var/aegir/platforms + drupal_9.3.13 + 2022.05.19
            # = /var/aegir/platforms/drupal_9.3.13_2022.05.19
            path=$(get_deployed_platform_path "$basepath" "$basename" "$DEPLOY_DATE")
            sanitized_platform_name=$(sanitize_platform_name "${basename}_${DEPLOY_DATE}")
            echo "Running aegir provision on $hostname: ..."
            ssh "$user"@"$hostname" drush --root="$path" --context_type='platform' provision-save @platform_"$sanitized_platform_name"
            ssh "$user"@"$hostname" drush @hostmaster hosting-import @platform_"$sanitized_platform_name"
        fi
    done
}

# /var/aegir/platforms + drupal_9.3.13 + 2022.05.19
# = /var/aegir/platforms/drupal_9.3.13_2022.05.19
function get_deployed_platform_path() {
    local path="$1"
    local basename="$2"
    local cur_date="$3"
    if [ -z "$cur_date" ] ; then
        cur_date=$(date +'%Y.%m.%d')
    fi
    echo "${path}/${basename}_${cur_date}"
}

# drupal_9.3.13_2022.05.19
# = drupal931320220519
function sanitize_platform_name() {
    local name="$1"
    # escape the name for sed first
    name=$(echo "$name" | sed 's/[-_.]//g;')
    echo "$name"
}
